#include "fic.h"

#include <stdlib.h>
#include <string.h>

#include "../coding/puncture.h"
#include "../coding/viterbi_decode.h"
#include "../coding/energy_dispersal.h"

fic *new_fic(void)
{
  fic *ret = calloc(1, sizeof(fic));
  if (ret == NULL) abort();

  ret->energy = new_energy_dispersal(768/8);

  return ret;
}

void fic_unpuncture(int *in, int *out)
{
  int i;

  /* first 21 block of 128 bits: puncturing pattern 16 */
  for (i = 0; i < 21 * 4; i++) {
    unpuncture(16, in, out);
    in += 24;
    out += 32;
  }

  /* next 3 block of 128 bits: puncturing pattern 15 */
  for (i = 0; i < 3 * 4; i++) {
    unpuncture(15, in, out);
    in += 23;
    out += 32;
  }

  /* last 24 bits, puncturing patter: 1100 1100 1100 1100 1100 1100 */
  for (i = 0; i < 6; i++) {
    *out++ = *in++;
    *out++ = *in++;
    *out++ = -128;
    *out++ = -128;
  }
}

void receive_fic_bits(fic *fic, int *bits)
{
  char curfic[768/8+1];
  int polys[4] = { 0133, 0171, 0145, 0133 };

  memcpy(&fic->bits[fic->pos], bits, 768*sizeof(int));
  fic->pos += 768;
  if (fic->pos != 2304)
    return;
  fic->pos = 0;
  fic_unpuncture(fic->bits, fic->unpunctured);
  memset(curfic, 0, sizeof(curfic));
  viterbi_decode(curfic, fic->unpunctured, 3072+24, 7, polys, 4);
  do_energy_dispersal(fic->energy, curfic);

  int i;
printf("YO ");
  for (i = 0; i < 768/8+1; i++) { printf(" %2.2x", (unsigned char)curfic[i]); if (i % 32 == 31) printf("|"); }
  printf("\n");
}
