#include "fft.h"

#include <complex.h>
#include <fftw3.h>
#include <stdlib.h>
#include <string.h>

typedef struct {
  complex *in;
  complex *out;
  fftw_plan p_fft;
  fftw_plan p_ifft;
  int size;
} _fft;

fft_data *init_fft(int fft_size)
{
  _fft *f;

  f = malloc(sizeof(_fft));

  f->in = fftw_malloc(sizeof(complex) * fft_size);
  f->out = fftw_malloc(sizeof(complex) * fft_size);
  if (f->in == NULL || f->out == NULL) abort();

  f->p_fft = fftw_plan_dft_1d(fft_size, f->in, f->out,
                              FFTW_FORWARD, FFTW_ESTIMATE);
  f->p_ifft = fftw_plan_dft_1d(fft_size, f->in, f->out,
                               FFTW_BACKWARD, FFTW_ESTIMATE);

  f->size = fft_size;

  memset(f->in, 0, sizeof(complex) * fft_size);
  fftw_execute(f->p_fft);
  fftw_execute(f->p_ifft);

  return (fft_data *)f;
}

void fft(fft_data *_f, complex *in, complex *out)
{
  _fft *f = _f;
  memcpy(f->in, in, f->size * sizeof(complex));
  fftw_execute(f->p_fft);
  memcpy(out, f->out, f->size * sizeof(complex));
}

void ifft(fft_data *_f, complex *in, complex *out)
{
  _fft *f = _f;
  memcpy(f->in, in, f->size * sizeof(complex));
  fftw_execute(f->p_ifft);
  memcpy(out, f->out, f->size * sizeof(complex));
}

void free_fft(fft_data *_f)
{
  _fft *f = _f;
  fftw_destroy_plan(f->p_fft);
  fftw_destroy_plan(f->p_ifft);
  fftw_free(f->in);
  fftw_free(f->out);
  free(f);
}
