#include "file.h"

#include <stdlib.h>
#include <stdint.h>
#include <string.h>

file_io *new_input_file(char *filename)
{
  file_io *ret = NULL;

  ret = calloc(1, sizeof(file_io));
  if (ret == NULL) { fprintf(stderr, "out of memory\n"); goto err; }

  ret->f = fopen(filename, "r");
  if (ret->f == NULL) {
    perror(filename);
    goto err;
  }

  return ret;

err:
  if (ret != NULL && ret->f != NULL) fclose(ret->f);
  free(ret);
  return NULL;
}

void close_input(file_io *f)
{
  fclose(f->f);
  free(f->back_buffer);
  free(f);
}

int get_samples(file_io *f, complex *out, int count)
{
  int a, b, c, d;
  int i;
  int16_t re, im;

  i = 0;

  if (f->back_buffer_pos != f->back_buffer_size) {
    while (i < count && f->back_buffer_pos != f->back_buffer_size) {
      if (out != NULL) out[i] = f->back_buffer[f->back_buffer_pos];
      f->back_buffer_pos++;
      i++;
    }

    if (f->back_buffer_pos != f->back_buffer_size)
      return i;

    free(f->back_buffer);
    f->back_buffer = NULL;
    f->back_buffer_pos = 0;
    f->back_buffer_size = 0;
  }

  for (; i < count; i++) {
    a = fgetc(f->f);
    b = fgetc(f->f);
    c = fgetc(f->f);
    d = fgetc(f->f);
    if (a == EOF || b == EOF || c == EOF || d == EOF)
      return i;
    re = a | (b << 8);
    im = c | (d << 8);
    if (out != NULL) out[i] = re / 32768. + im / 32768. * I;
  }

  return i;
}

void put_back_samples(file_io *f, complex *in, int count)
{
  if (f->back_buffer_pos != f->back_buffer_size) {
    /* can't put back more than what was put back before */
    if (count > f->back_buffer_pos) abort();
    f->back_buffer_pos -= count;
    memcpy(&f->back_buffer[f->back_buffer_pos], in, count * sizeof(complex));
    return;
  }
  free(f->back_buffer);
  f->back_buffer = malloc(count * sizeof(complex));
  if (f->back_buffer == NULL) abort();
  memcpy(f->back_buffer, in, count * sizeof(complex));
  f->back_buffer_pos = 0;
  f->back_buffer_size = count;
}
